@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2013 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -35,6 +35,10 @@
 #include "util-unittest.h"
 #include "util-debug.h"
 
+#include "pkt-var.h"
+#include "host.h"
+#include "util-profiling.h"
+
 /* prototypes for the "ipv4-csum" rule keyword */
 int DetectIPV4CsumMatch(ThreadVars *, DetectEngineThreadCtx *, Packet *,
                         Signature *, SigMatch *);
@@ -237,13 +241,13 @@ int DetectIPV4CsumMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
         return cd->valid;
     }
 
-    if (p->ip4vars.comp_csum == -1)
-        p->ip4vars.comp_csum = IPV4CalculateChecksum((uint16_t *)p->ip4h,
-                                                     IPV4_GET_HLEN(p));
+    if (p->level3_comp_csum == -1)
+        p->level3_comp_csum = IPV4CalculateChecksum((uint16_t *)p->ip4h,
+                                                    IPV4_GET_HLEN(p));
 
-    if (p->ip4vars.comp_csum == p->ip4h->ip_csum && cd->valid == 1)
+    if (p->level3_comp_csum == p->ip4h->ip_csum && cd->valid == 1)
         return 1;
-    else if (p->ip4vars.comp_csum != p->ip4h->ip_csum && cd->valid == 0)
+    else if (p->level3_comp_csum != p->ip4h->ip_csum && cd->valid == 0)
         return 1;
     else
         return 0;
@@ -331,14 +335,14 @@ int DetectTCPV4CsumMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
         return cd->valid;
     }
 
-    if (p->tcpvars.comp_csum == -1)
-        p->tcpvars.comp_csum = TCPCalculateChecksum(p->ip4h->s_ip_addrs,
-                                                 (uint16_t *)p->tcph,
-                                                 (p->payload_len + TCP_GET_HLEN(p)));
+    if (p->level4_comp_csum == -1)
+        p->level4_comp_csum = TCPCalculateChecksum(p->ip4h->s_ip_addrs,
+                                                   (uint16_t *)p->tcph,
+                                                   (p->payload_len + TCP_GET_HLEN(p)));
 
-    if (p->tcpvars.comp_csum == p->tcph->th_sum && cd->valid == 1)
+    if (p->level4_comp_csum == p->tcph->th_sum && cd->valid == 1)
         return 1;
-    else if (p->tcpvars.comp_csum != p->tcph->th_sum && cd->valid == 0)
+    else if (p->level4_comp_csum != p->tcph->th_sum && cd->valid == 0)
         return 1;
     else
         return 0;
@@ -426,14 +430,14 @@ int DetectTCPV6CsumMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
         return cd->valid;
     }
 
-    if (p->tcpvars.comp_csum == -1)
-        p->tcpvars.comp_csum = TCPV6CalculateChecksum(p->ip6h->s_ip6_addrs,
-                                                   (uint16_t *)p->tcph,
-                                                   (p->payload_len + TCP_GET_HLEN(p)));
+    if (p->level4_comp_csum == -1)
+        p->level4_comp_csum = TCPV6CalculateChecksum(p->ip6h->s_ip6_addrs,
+                                                     (uint16_t *)p->tcph,
+                                                     (p->payload_len + TCP_GET_HLEN(p)));
 
-    if (p->tcpvars.comp_csum == p->tcph->th_sum && cd->valid == 1)
+    if (p->level4_comp_csum == p->tcph->th_sum && cd->valid == 1)
         return 1;
-    else if (p->tcpvars.comp_csum != p->tcph->th_sum && cd->valid == 0)
+    else if (p->level4_comp_csum != p->tcph->th_sum && cd->valid == 0)
         return 1;
     else
         return 0;
@@ -521,15 +525,15 @@ int DetectUDPV4CsumMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
         return cd->valid;
     }
 
-    if (p->udpvars.comp_csum == -1)
-        p->udpvars.comp_csum = UDPV4CalculateChecksum(p->ip4h->s_ip_addrs,
-                                                   (uint16_t *)p->udph,
-                                                   (p->payload_len +
-                                                    UDP_HEADER_LEN) );
+    if (p->level4_comp_csum == -1)
+        p->level4_comp_csum = UDPV4CalculateChecksum(p->ip4h->s_ip_addrs,
+                                                     (uint16_t *)p->udph,
+                                                     (p->payload_len +
+                                                      UDP_HEADER_LEN) );
 
-    if (p->udpvars.comp_csum == p->udph->uh_sum && cd->valid == 1)
+    if (p->level4_comp_csum == p->udph->uh_sum && cd->valid == 1)
         return 1;
-    else if (p->udpvars.comp_csum != p->udph->uh_sum && cd->valid == 0)
+    else if (p->level4_comp_csum != p->udph->uh_sum && cd->valid == 0)
         return 1;
     else
         return 0;
@@ -617,15 +621,15 @@ int DetectUDPV6CsumMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
         return cd->valid;
     }
 
-    if (p->udpvars.comp_csum == -1)
-        p->udpvars.comp_csum = UDPV6CalculateChecksum(p->ip6h->s_ip6_addrs,
-                                                   (uint16_t *)p->udph,
-                                                   (p->payload_len +
-                                                    UDP_HEADER_LEN) );
+    if (p->level4_comp_csum == -1)
+        p->level4_comp_csum = UDPV6CalculateChecksum(p->ip6h->s_ip6_addrs,
+                                                     (uint16_t *)p->udph,
+                                                     (p->payload_len +
+                                                      UDP_HEADER_LEN) );
 
-    if (p->udpvars.comp_csum == p->udph->uh_sum && cd->valid == 1)
+    if (p->level4_comp_csum == p->udph->uh_sum && cd->valid == 1)
         return 1;
-    else if (p->udpvars.comp_csum != p->udph->uh_sum && cd->valid == 0)
+    else if (p->level4_comp_csum != p->udph->uh_sum && cd->valid == 0)
         return 1;
     else
         return 0;
@@ -713,14 +717,14 @@ int DetectICMPV4CsumMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
         return cd->valid;
     }
 
-    if (p->icmpv4vars.comp_csum == -1)
-        p->icmpv4vars.comp_csum = ICMPV4CalculateChecksum((uint16_t *)p->icmpv4h,
-                                                       ntohs(IPV4_GET_RAW_IPLEN(p->ip4h)) -
-                                                       IPV4_GET_RAW_HLEN(p->ip4h) * 4);
+    if (p->level4_comp_csum == -1)
+        p->level4_comp_csum = ICMPV4CalculateChecksum((uint16_t *)p->icmpv4h,
+                                                      ntohs(IPV4_GET_RAW_IPLEN(p->ip4h)) -
+                                                      IPV4_GET_RAW_HLEN(p->ip4h) * 4);
 
-    if (p->icmpv4vars.comp_csum == p->icmpv4h->checksum && cd->valid == 1)
+    if (p->level4_comp_csum == p->icmpv4h->checksum && cd->valid == 1)
         return 1;
-    else if (p->icmpv4vars.comp_csum != p->icmpv4h->checksum && cd->valid == 0)
+    else if (p->level4_comp_csum != p->icmpv4h->checksum && cd->valid == 0)
         return 1;
     else
         return 0;
@@ -810,14 +814,14 @@ int DetectICMPV6CsumMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
         return cd->valid;
     }
 
-    if (p->icmpv6vars.comp_csum == -1)
-        p->icmpv6vars.comp_csum = ICMPV6CalculateChecksum(p->ip6h->s_ip6_addrs,
-                                                          (uint16_t *)p->icmpv6h,
-                                                          GET_PKT_LEN(p) - ((uint8_t *)p->icmpv6h - GET_PKT_DATA(p)));
+    if (p->level4_comp_csum == -1)
+        p->level4_comp_csum = ICMPV6CalculateChecksum(p->ip6h->s_ip6_addrs,
+                                                      (uint16_t *)p->icmpv6h,
+                                                      GET_PKT_LEN(p) - ((uint8_t *)p->icmpv6h - GET_PKT_DATA(p)));
 
-    if (p->icmpv6vars.comp_csum == p->icmpv6h->csum && cd->valid == 1)
+    if (p->level4_comp_csum == p->icmpv6h->csum && cd->valid == 1)
         return 1;
-    else if (p->icmpv6vars.comp_csum != p->icmpv6h->csum && cd->valid == 0)
+    else if (p->level4_comp_csum != p->icmpv6h->csum && cd->valid == 0)
         return 1;
     else
         return 0;
@@ -1514,7 +1518,6 @@ int DetectCsumICMPV6Test01(void)
 
     PacketCopyData(p, pkt, sizeof(pkt));
 
-
     memset(&tv, 0, sizeof(tv));
     memset(&dtv, 0, sizeof(dtv));
 
@@ -1526,7 +1529,7 @@ int DetectCsumICMPV6Test01(void)
         printf("DetectEngineCtxInit failure\n");
         goto end;
     }
-    de_ctx->mpm_matcher = MPM_AC;
+    de_ctx->mpm_matcher = DEFAULT_MPM;
     de_ctx->flags |= DE_QUIET;
 
     s = de_ctx->sig_list = SigInit(de_ctx, "alert ip any any -> any any "
@@ -1559,6 +1562,7 @@ int DetectCsumICMPV6Test01(void)
     }
 
     StreamTcpFreeConfig(TRUE);
+    PACKET_RECYCLE(p);
     FlowShutdown();
 
     SCFree(p);
