@@ -29,6 +29,7 @@
 #include "decode.h"
 
 #include "detect.h"
+#include "detect-engine.h"
 #include "detect-parse.h"
 #include "detect-content.h"
 #include "detect-uricontent.h"
@@ -137,7 +138,7 @@ static int DetectWithinSetup(DetectEngineCtx *de_ctx, Signature *s, char *within
         goto end;
     }
     if (str[0] != '-' && isalpha((unsigned char)str[0])) {
-        SigMatch *bed_sm = DetectByteExtractRetrieveSMVar(str, s, SigMatchListSMBelongsTo(s, pm));
+        SigMatch *bed_sm = DetectByteExtractRetrieveSMVar(str, s);
         if (bed_sm == NULL) {
             SCLogError(SC_ERR_INVALID_SIGNATURE, "unknown byte_extract var "
                        "seen in within - %s\n", str);
@@ -244,6 +245,38 @@ end:
     return result;
 }
 
+static int DetectWithinTestVarSetup(void)
+{
+    DetectEngineCtx *de_ctx = NULL;
+    int result = 0;
+    char sig[] = "alert tcp any any -> any any ( "
+        "msg:\"test rule\"; "
+        "content:\"abc\"; "
+        "http_client_body; "
+        "byte_extract:2,0,somevar,relative; "
+        "content:\"def\"; "
+        "within:somevar; "
+        "http_client_body; "
+        "sid:4; rev:1;)";
+
+    de_ctx = DetectEngineCtxInit();
+    if (de_ctx == NULL) {
+        goto end;
+    }
+    de_ctx->sig_list = SigInit(de_ctx, sig);
+    if (de_ctx->sig_list == NULL) {
+        goto end;
+    }
+
+    result = 1;
+
+end:
+    SigGroupCleanup(de_ctx);
+    SigCleanSignatures(de_ctx);
+    DetectEngineCtxFree(de_ctx);
+
+    return result;
+}
 
 #endif /* UNITTESTS */
 
@@ -251,5 +284,6 @@ void DetectWithinRegisterTests(void) {
     #ifdef UNITTESTS
     UtRegisterTest("DetectWithinTestPacket01", DetectWithinTestPacket01, 1);
     UtRegisterTest("DetectWithinTestPacket02", DetectWithinTestPacket02, 1);
+    UtRegisterTest("DetectWithinTestVarSetup", DetectWithinTestVarSetup, 1);
     #endif /* UNITTESTS */
 }
