@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2014 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -32,6 +32,7 @@
 #define TM_FLAG_DECODE_TM       0x02
 #define TM_FLAG_STREAM_TM       0x04
 #define TM_FLAG_DETECT_TM       0x08
+#define TM_FLAG_LOGAPI_TM       0x10 /**< TM is run by Log API */
 
 typedef struct TmModule_ {
     char *name;
@@ -60,35 +61,6 @@ typedef struct TmModule_ {
 
 TmModule tmm_modules[TMM_SIZE];
 
-/** Global structure for Output Context */
-typedef struct LogFileCtx_ {
-    FILE *fp;
-    /** It will be locked if the log/alert
-     * record cannot be written to the file in one call */
-    SCMutex fp_mutex;
-
-    /** The name of the file */
-    char *filename;
-
-    /**< Used by some alert loggers like the unified ones that append
-     * the date onto the end of files. */
-    char *prefix;
-
-    /** Generic size_limit and size_current
-     * They must be common to the threads accesing the same file */
-    uint64_t size_limit;    /**< file size limit */
-    uint64_t size_current;  /**< file current size */
-
-    /* Alerts on the module (not on the file) */
-    uint64_t alerts;
-    /* flag to avoid multiple threads printing the same stats */
-    uint8_t flags;
-} LogFileCtx;
-
-/* flags for LogFileCtx */
-#define LOGFILE_HEADER_WRITTEN 0x01
-#define LOGFILE_ALERTS_PRINTED 0x02
-
 /**
  * Structure that output modules use to maintain private data.
  */
@@ -101,11 +73,9 @@ typedef struct OutputCtx_ {
     void (*DeInit)(struct OutputCtx_ *);
 } OutputCtx;
 
-LogFileCtx *LogFileNewCtx();
-int LogFileFreeCtx(LogFileCtx *);
-
-TmModule *TmModuleGetByName(char *name);
+TmModule *TmModuleGetByName(const char *name);
 TmModule *TmModuleGetById(int id);
+int TmModuleGetIdByName(const char *name);
 int TmModuleGetIDForTM(TmModule *tm);
 TmEcode TmModuleRegister(char *name, int (*module_func)(ThreadVars *, Packet *, void *));
 void TmModuleDebugList(void);
