@@ -359,8 +359,8 @@ int FileStore(File *ff) {
  *  \param ff The file to store
  *  \param txid the tx id
  */
-int FileSetTx(File *ff, uint16_t txid) {
-    SCLogDebug("ff %p txid %"PRIu16, ff, txid);
+int FileSetTx(File *ff, uint64_t txid) {
+    SCLogDebug("ff %p txid %"PRIu64, ff, txid);
     if (ff != NULL)
         ff->txid = txid;
     SCReturnInt(0);
@@ -643,7 +643,7 @@ void FileDisableStoring(Flow *f, uint8_t direction) {
     else
         f->flags |= FLOW_FILE_NO_STORE_TC;
 
-    FileContainer *ffc = AppLayerGetFilesFromFlow(f, direction);
+    FileContainer *ffc = AppLayerParserGetFiles(f->proto, f->alproto, f->alstate, direction);
     if (ffc != NULL) {
         for (ptr = ffc->head; ptr != NULL; ptr = ptr->next) {
             /* if we're already storing, we'll continue */
@@ -674,7 +674,7 @@ void FileDisableMagic(Flow *f, uint8_t direction) {
     else
         f->flags |= FLOW_FILE_NO_MAGIC_TC;
 
-    FileContainer *ffc = AppLayerGetFilesFromFlow(f, direction);
+    FileContainer *ffc = AppLayerParserGetFiles(f->proto, f->alproto, f->alstate, direction);
     if (ffc != NULL) {
         for (ptr = ffc->head; ptr != NULL; ptr = ptr->next) {
             SCLogDebug("disabling magic for file %p from direction %s",
@@ -704,7 +704,7 @@ void FileDisableMd5(Flow *f, uint8_t direction) {
     else
         f->flags |= FLOW_FILE_NO_MD5_TC;
 
-    FileContainer *ffc = AppLayerGetFilesFromFlow(f, direction);
+    FileContainer *ffc = AppLayerParserGetFiles(f->proto, f->alproto, f->alstate, direction);
     if (ffc != NULL) {
         for (ptr = ffc->head; ptr != NULL; ptr = ptr->next) {
             SCLogDebug("disabling md5 for file %p from direction %s",
@@ -742,7 +742,7 @@ void FileDisableFilesize(Flow *f, uint8_t direction) {
     else
         f->flags |= FLOW_FILE_NO_SIZE_TC;
 
-    FileContainer *ffc = AppLayerGetFilesFromFlow(f, direction);
+    FileContainer *ffc = AppLayerParserGetFiles(f->proto, f->alproto, f->alstate, direction);
     if (ffc != NULL) {
         for (ptr = ffc->head; ptr != NULL; ptr = ptr->next) {
             SCLogDebug("disabling size tracking for file %p from direction %s",
@@ -783,14 +783,14 @@ void FileDisableStoringForFile(File *ff) {
  *  \param direction flow direction
  *  \param tx_id transaction id
  */
-void FileDisableStoringForTransaction(Flow *f, uint8_t direction, uint16_t tx_id) {
+void FileDisableStoringForTransaction(Flow *f, uint8_t direction, uint64_t tx_id) {
     File *ptr = NULL;
 
     DEBUG_ASSERT_FLOW_LOCKED(f);
 
     SCEnter();
 
-    FileContainer *ffc = AppLayerGetFilesFromFlow(f, direction);
+    FileContainer *ffc = AppLayerParserGetFiles(f->proto, f->alproto, f->alstate, direction);
     if (ffc != NULL) {
         for (ptr = ffc->head; ptr != NULL; ptr = ptr->next) {
             if (ptr->txid == tx_id) {
